@@ -1,27 +1,8 @@
+from django.db.models import signals
 from django.db.models.fields import CharField, FloatField, CharField
 
 from django_measurement import measure
 
-
-class MeasureField(CharField):
-    def __init__(self, *args, **kwargs):
-        kwargs['max_length'] = kwargs.get('max_length', 10)
-        kwargs['choices'] = kwargs.get('choices', zip(measure.__all__, measure.__all__))
-        super(MeasureField, self).__init__(*args, **kwargs)
-
-    def get_internal_type(self):
-        return 'CharField'
-
-
-class OriginalUnitField(CharField):
-    def __init__(self, *args, **kwargs):
-        kwargs['max_length'] = kwargs.get('max_length', 20)
-        super(OriginalUnitField, self).__init__(*args, **kwargs)
-
-    def get_internal_type(self):
-        return 'CharField'
-
-
 class MeasurementFieldDescriptor(object):
     def __init__(self, field, measurement_field_name, original_unit_field_name, measure_field_name):
         self.field = field
@@ -29,17 +10,14 @@ class MeasurementFieldDescriptor(object):
         self.original_unit_field_name = original_unit_field_name
         self.measure_field_name = measure_field_name
 
-    def _get_measure(self, instance):
-        return getattr(
-            measure,
-            getattr(instance, self.measure_field_name)
-        )
-
     def __get__(self, instance, instance_type=None):
         if instance is None:
             return self
 
-        instance_measure = self._get_measure(instance)
+        instance_measure = getattr(
+            measure,
+            getattr(instance, self.measure_field_name)
+        )
         measurement_value = getattr(
             instance,
             self.measurement_field_name,
@@ -70,11 +48,23 @@ class MeasurementField(FloatField):
             **kwargs
         ):
         super(MeasurementField, self).__init__(*args, **kwargs)
+        self.original_unit_field_name = '%s_unit'
+        self.measure_field_name = '%s_measure'
+        self.measurement_field_name = '%s_value'
+
+    def get_original_unit_field_name(self):
+        return self.original_unit_field_name % self.name
+
+    def get_measure_field_name(self):
+        return self.measure_field_name % self.name
+
+    def get_measurement_field_name(self):
+        return self.measurement_field_name % self.name
 
     def contribute_to_class(self, cls, name):
         self.name = name
 
-        original_unit_field_name = '%s_unit' % self.name
+        original_unit_field_name = self.get_original_unit_field_name()
         self.original_unit_field = CharField(
             editable=False,
             blank=True,
@@ -83,7 +73,7 @@ class MeasurementField(FloatField):
         )
         cls.add_to_class(original_unit_field_name, self.original_unit_field)
 
-        measure_field_name = '%s_measure' % self.name
+        measure_field_name = self.get_measure_field_name()
         self.measure_field = CharField(
             editable=False,
             blank=True,
@@ -92,7 +82,7 @@ class MeasurementField(FloatField):
         )
         cls.add_to_class(measure_field_name, self.measure_field)
 
-        measurement_field_name = '%s_value' % self.name
+        measurement_field_name = self.get_measurement_field_name()
         self.measurement_field = CharField(
             editable=False,
             blank=True,
@@ -108,20 +98,20 @@ class MeasurementField(FloatField):
             measure_field_name=measure_field_name,
         )
 
+        signals.pre_init.connect(self.instance_pre_init, sender=cls, weak=False)
+
         setattr(cls, self.name, field)
 
-    def __get__(self, instance, type=None):
-        if instance is None:
-            return self
-        return self.instance_class(instance, self)
-
-    def __set__(self, instance, value):
-        if not isinstance(value, measure.MeasureBase):
-            raise TypeError('')
+    def instance_pre_init(self, signal, sender, args, kwargs, **_kwargs):
+        if self.name in kwargs:
+            value = kwargs.pop(self.name)
+            kwargs[self.get_original_unit_field_name()] = value._default_unit
+            kwargs[self.get_measure_field_name()] = value.__class__.__name__
+            kwargs[self.get_measurement_field_name()] = getattr(value, value.STANDARD_UNIT, 0)
 
 
 try:
     from south.modelsinspector import add_introspection_rules
-    add_introspection_rules([], ["^django_measurement\.fields\.(MeasuremeField|MeasurementField|OriginalUnitField)"])
+    add_introspection_rules([], ["^django_measurement\.fields\.MeasurementField"])
 except ImportError:
     pass
